@@ -4,19 +4,21 @@
 For usage, see README.md.
 '''
 
-import argparse
 import logging
 import mimetypes
 import os
+import requests
 import socket
 import sys
 from threading import Timer
+import time
 import webbrowser
 
 from flask import (Flask, render_template, send_from_directory,
                    request, jsonify, Response)
 
 import util
+import argparser
 
 
 def determine_path():
@@ -35,11 +37,6 @@ def is_hot_reload():
     """In debug mode, Werkzeug reloads the app on any changes."""
     return os.environ.get('WERKZEUG_RUN_MAIN')
 
-parser = argparse.ArgumentParser(description='Run webdiff.')
-parser.add_argument('--port', '-p', type=int, help="Port to run webdiff on.", default=-1)
-parser.add_argument('a', type=str, help="'Left' file to diff")
-parser.add_argument('b', type=str, help="'Right' file to diff")
-args = parser.parse_args()
 
 class Config:
     pass
@@ -61,15 +58,23 @@ if app.config['TESTING'] or app.config['DEBUG']:
     handler.setFormatter(formatter)
 
     app.logger.addHandler(handler)
-    for logname in ['github', '']:
+    for logname in ['']:
         log = logging.getLogger(logname)
         log.setLevel(logging.DEBUG)
         log.addHandler(handler)
+    logging.getLogger('github').setLevel(logging.ERROR)
 else:
     # quiet down werkzeug -- no need to log every request.
     logging.getLogger('werkzeug').setLevel(logging.ERROR)
 
 
+LAST_REQUEST_MS = 0
+@app.before_request
+def update_last_request_ms():
+    global LAST_REQUEST_MS
+    LAST_REQUEST_MS = time.time() * 1000
+
+
 @app.route("/<side>/get_contents", methods=['POST'])
 def get_contents(side):
     assert side in ('a', 'b')
@@ -156,9 +161,8 @@ def favicon():
                                mimetype='image/vnd.microsoft.icon')
 
 
-@app.route('/kill', methods=['POST'])
-def kill():
-    if 'STAY_RUNNING' in app.config: return
+@app.route('/seriouslykill', methods=['POST'])
+def seriouslykill():
     func = request.environ.get('werkzeug.server.shutdown')
     if func is None:
         raise RuntimeError('Not running with the Werkzeug Server')
@@ -166,6 +170,22 @@ def kill():
     return "Shutting down..."
 
 
+@app.route('/kill', methods=['POST'])
+def kill():
+    if 'STAY_RUNNING' in app.config: return
+
+    last_ms = LAST_REQUEST_MS
+    def shutdown():
+        if LAST_REQUEST_MS <= last_ms:  # subsequent requests abort shutdown
+            requests.post('http://localhost:%d/seriouslykill' % PORT)
+        else:
+            pass
+
+    Timer(0.5, shutdown).start()
+    
+    return "Shutting down..."
+
+
 def open_browser():
     global PORT
     if not 'NO_OPEN_BROWSER' in app.config:
@@ -175,24 +195,14 @@ def open_browser():
             webbrowser.open_new_tab('http://localhost:%s' % PORT)
 
 
-def err_and_die(err, code=1):
-    sys.stderr.write('ERROR: %s\n' % err.strip())
-    sys.exit(code)
-
-
 def usage_and_die():
-    sys.stderr.write(
-'''Usage: webdiff <left_dir> <right_dir>
-       webdiff <left_file> <right_file>
-
-Or run "git webdiff" from a git repository.
-''')
+    sys.stderr.write(argparser.USAGE)
     sys.exit(1)
 
 
-def pick_a_port():
-    if args.port != -1:
-        return args.port
+def pick_a_port(args):
+    if 'port' in args != -1:
+        return args['port']
 
     if os.environ.get('WEBDIFF_PORT'):
         return int(os.environ.get('WEBDIFF_PORT'))
@@ -212,51 +222,24 @@ def abs_path(path):
         return os.path.join(os.getcwd(), path)
 
 
-def shim_for_file_diff(a_file, b_file):
-    '''Sets A_DIR, B_DIR and DIFF to do a one-file diff.'''
-    global A_DIR, B_DIR, DIFF
-    A_DIR = os.path.dirname(a_file)
-    a_file = os.path.basename(a_file)
-    B_DIR = os.path.dirname(b_file)
-    b_file = os.path.basename(b_file)
-    DIFF = util.annotate_file_pairs([{
-             'a': a_file,
-             'b': b_file,
-             'idx': 0,
-             'path': a_file,
-             'type': 'change'}],  # 'change' is the only likely case.
-             A_DIR, B_DIR)
-
-
 def run():
     global A_DIR, B_DIR, DIFF, PORT
-    A_DIR = abs_path(args.a)
-    B_DIR = abs_path(args.b)
+    try:
+        parsed_args = argparser.parse(sys.argv[1:])
+    except argparser.UsageError as e:
+        sys.stderr.write('Error: %s\n\n' % e.message)
+        usage_and_die()
 
-    if not os.path.exists(A_DIR):
-        err_and_die("'%s' doesn't exist" % A_DIR)
-
-    if not os.path.exists(B_DIR):
-        err_and_die("'%s' doesn't exist" % B_DIR)
-
-    if os.path.isdir(A_DIR) and not os.path.isdir(B_DIR):
-        err_and_die("'%s' is a directory but '%s' is not" % (A_DIR, B_DIR))
-
-    if not os.path.isdir(A_DIR) and os.path.isdir(B_DIR):
-        err_and_die("'%s' is a directory but '%s' is not" % (B_DIR, A_DIR))
+    A_DIR, B_DIR, DIFF = util.diff_for_args(parsed_args)
 
     if app.config['TESTING'] or app.config['DEBUG']:
         sys.stderr.write('Diffing:\nA: %s\nB: %s\n\n' % (A_DIR, B_DIR))
 
-    PORT = pick_a_port()
+    PORT = pick_a_port(parsed_args)
 
     sys.stderr.write('''Serving diffs on http://localhost:%s
 Close the browser tab or hit Ctrl-C when you're done.
 ''' % PORT)
-    if os.path.isdir(A_DIR):
-        DIFF = util.find_diff(A_DIR, B_DIR)
-    else:
-        shim_for_file_diff(A_DIR, B_DIR)
     logging.info('Diff: %r', DIFF)
     Timer(0.1, open_browser).start()
     app.run(host='0.0.0.0', port=PORT)
